# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

class Migration(DataMigration):
    
    def forwards(self, orm):
        "Write your forwards methods here."
        # create the plpgsql language needed
        db.execute("CREATE LANGUAGE plpgsql;")

        db.execute("""
            CREATE OR REPLACE FUNCTION split_timestamp()
              RETURNS trigger AS
            $BODY$
            DECLARE
                dd VARCHAR;
                dt VARCHAR;
            BEGIN
               dd := to_char(NEW.calldate, 'YYYYMMDD');
               dt := to_char(NEW.calldate, 'HH24:MI:SS');

               UPDATE asterisk_cdr 
               SET calldate_date = to_date(dd, 'YYYYMMDD'),
                   calldate_time = to_timestamp(dt, 'HH24:MI:SS')
               WHERE id = NEW.id;
               RETURN NEW;
            END;
            $BODY$
              LANGUAGE 'plpgsql' VOLATILE
              COST 100;
        """)

        db.execute("""
        CREATE TRIGGER on_insert_split_ts
          AFTER INSERT
          ON asterisk_cdr
          FOR EACH ROW
          EXECUTE PROCEDURE split_timestamp();
        """)


        db.execute("""update asterisk_cdr set 
                      calldate_date = calldate, 
                      calldate_time = to_timestamp(to_char(calldate, 'HH24:MI:SS'), 'HH24:MI:SS')""")

        db.execute("""update asterisk_record set 
                      ts_date = ts, 
                      ts_time = to_timestamp(to_char(ts, 'HH24:MI:SS'), 'HH24:MI:SS')""")


    def backwards(self, orm):
        "Write your backwards methods here."
        db.execute("DROP FUNCTION split_timestamp();")
        db.execute("DROP TRIGGER on_insert_split_ts ON asterisk_cdr;")
        db.execute("DROP LANGUAGE plpgsql;")
    
    models = {
        'asterisk.asteriskaddressbook': {
            'Meta': {'object_name': 'AsteriskAddressBook'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '15'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_internal': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'phone_number': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'asterisk.asteriskprofile': {
            'Meta': {'object_name': 'AsteriskProfile'},
            'description': ('django.db.models.fields.CharField', [], {'max_length': '25', 'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'internal_number': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True', 'blank': 'True'})
        },
        'asterisk.cdr': {
            'Meta': {'object_name': 'Cdr'},
            'accountcode': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'amaflags': ('django.db.models.fields.IntegerField', [], {}),
            'billsec': ('django.db.models.fields.IntegerField', [], {}),
            'calldate': ('django.db.models.fields.DateTimeField', [], {}),
            'calldate_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'calldate_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'}),
            'channel': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'clid': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'dcontext': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'disposition': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'dst': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'dstchannel': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'duration': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lastapp': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'lastdata': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'src': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'uniqueid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'userfield': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'asterisk.record': {
            'Meta': {'object_name': 'Record'},
            'callee': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'caller': ('django.db.models.fields.CharField', [], {'max_length': '80'}),
            'recall': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'recin': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'record': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'recout': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'ts': ('django.db.models.fields.DateTimeField', [], {}),
            'ts_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'ts_time': ('django.db.models.fields.TimeField', [], {'null': 'True', 'blank': 'True'})
        },
        'asterisk.settings': {
            'Meta': {'object_name': 'Settings'},
            'creator': ('django.db.models.ForeignKey', [], {'blank': 'True', 'related_name': "'asterisk_settings_creator'", 'null': 'True', 'to': "orm['auth.User']"}),
            'date_create': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'date_last_modify': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'last_modifier': ('django.db.models.ForeignKey', [], {'blank': 'True', 'related_name': "'asterisk_settings_last_modifier'", 'null': 'True', 'to': "orm['auth.User']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '20'}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '500', 'null': 'True', 'blank': 'True'})
        },
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        }
    }
    
    complete_apps = ['asterisk']
