# -*- coding: utf-8 -*-
import os,sys
from django.core import management;

import os, os.path, time
from datetime import timedelta
from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils import formats
from decimal import *

from jumbo.db.models import Settings as JSettings
from jumbo.utils.functions import get_settings
from jumbo.db.models import StatusModel, DateModel, UserModel

DCONTEXT = (
    ('check-password', _('CHECK PASSWORD')),
    ('default', _('DEFAULT')),
    ('from-gsm', _('FROM GSM')),
    ('from-internal', _('INTERNAL')),
    ('from-pstn', _('EXTERNAL')),
)

LAST_APP = (
    ('Dial', 'Dial'),
    ('Echo', 'Echo'),
    ('Goto', 'Go to'),
    ('GotoIf', 'Go to if'),
    ('GotoIfTime', 'Go to if time'),
    ('Hangup', 'Hangup'),
    ('Playback', 'Playback'),
    ('PlayTones', 'Play tones'),
)

DISPOSITION = (
    (0, ''),
    (1, ''),
    (2, ''),
    (4, ''),
    (8, ''),
)

AMAFLAGS = (
    (3, ''),
)

RECORDS_PATH = os.path.join(get_settings("asterisk.settings", "ASTERISK_PATH_DOCUMENTS"), 'records')

class Cdr (models.Model):
    calldate = models.DateTimeField(
    )
    calldate_date = models.DateField(
        blank = True, null = True,
        verbose_name = _("Call Date")
    )
    calldate_time = models.TimeField(
        blank = True, null = True,
        verbose_name = _("Call Time")
    )
    clid = models.CharField(
        max_length = 80,
        verbose_name = _('Caller'),
        help_text = 'Caller*ID with text',
    )
    src = models.CharField(
        max_length = 80,
        verbose_name = _('Caller Number'),
        help_text = 'Caller*ID number',
    )
    dst = models.CharField(
        max_length = 80,
        verbose_name = _('Destination'),
        help_text = 'Destination extension',
    )
    dcontext = models.CharField(
        max_length = 80,
        choices = DCONTEXT,
        verbose_name = _('Context'),
        help_text = 'Destination context',
    )
    channel = models.CharField(
        max_length = 80,
        verbose_name = _('Source Channel'),
        help_text = 'Channel used',
    )
    dstchannel = models.CharField(
        max_length = 80,
        verbose_name = _('Dest. Channel'),
        help_text = 'Destination channel if appropriate',
    )
    lastapp = models.CharField(
        max_length = 80,
        choices = LAST_APP,
        verbose_name = _('Last Application'),
        help_text = 'Last application if appropriate',
    )
    lastdata = models.CharField(
        max_length = 80,
        verbose_name = _('Last application data'),
        help_text = 'Last application data (arguments)',
    )
    duration = models.IntegerField(
        verbose_name = _('Total Call Time'),
        help_text = 'Total time in system, in seconds',
    )
    billsec = models.IntegerField(
        verbose_name = _('Totale Call Pay Time'),
        help_text = 'Total time call is up, in seconds',
    )
    disposition = models.CharField(
        max_length = 80,
        choices = DISPOSITION,
        verbose_name = _('Status'),
        help_text = 'What happened to the call: ANSWERED, NO ANSWER, BUSY, FAILED',
    )
    amaflags = models.IntegerField(
        choices = AMAFLAGS,
        verbose_name = '',
        help_text = '',
    )
    accountcode = models.CharField(
        max_length = 20,
        verbose_name = '',
        help_text = '',
    )
    uniqueid = models.CharField(
        max_length = 40,
        verbose_name = '',
        help_text = '',
    )
    userfield = models.CharField(
        max_length = 255,
        verbose_name = '',
        help_text = '',
    )
    callcost = models.DecimalField(
        max_digits=9,
        null=True, blank=True,
        decimal_places=2, verbose_name=_("Call Cost")
    )
    callcost_currency = models.CharField(
        max_length=5,
        null=True, blank=True,
    )

    class Meta:
        verbose_name = _("Phone Call")
        verbose_name_plural = _("Phone Calls")
        #ordering = ('-calldate',)
        permissions = (
            ("view_cdr", "Can view cdr"),
            ("view_callcost", "Can view Call Cost and report"),
        )

    def __unicode__(self):
        if self.clid:
            return u"%s (%s)" % (self.clid, self.get_nome_interno())
        return "Call"

    def get_nome_interno(self):
        nome = ""
        if self.clid.find("<") > -1:
            nome = self.clid.split("<")[0]
        return nome.strip()

    def get_prefix_eval(self):
        # Tolgo il numero che indica l'interno
        numero = self.dst[1:]
        # Tolgo gli eventuali 00 del prefisso internazione
        if numero[0:2] == '00': numero = numero[2:]

        # Ciclo la stringa del numero al contrario e per ogni ciclo controllo se esiste un prefisso con quel numero
        for x in range(len(numero)-1):
            n = numero[:-x]
            try:
               prefix = Prefix.objects.get(prefix = n)
               return prefix
            except Exception, Prefix.DoesNotExist:
               continue
        return None

    def get_cost_eval(self):
        # Cerco il prefisso del numero
        pref = self.get_prefix_eval()
        currency = None
        scatto_risposta = 0

        if pref:
            # E recupero il costo orario
            try:
                sing_cost = UnitCost.objects.get(
                                     fare__prefix = pref,
                                     hour_from__lt=self.calldate_time,
                                     hour_to__gt= self.calldate_time
                                     )
            except Exception, e:
                # Se non trovo il costo ritorno -1 in modo che possa segnalarlo come errore
                return None, None #print self.clid , self.id,  self.calldate_time
                #sing_cost = Decimal("0.2")

            scatto_risposta = sing_cost.fare.connection_fee
            currency = sing_cost.fare.symbol_currency
            sec_cost = sing_cost.cost / Decimal("60.0")
            sec_cost = sec_cost.quantize(Decimal('0.0001'),ROUND_HALF_UP)
        else:
            # Se non trovo il prefisso ritorno -1 in modo che possa segnalarlo come errore
            return None, None #sec_cost = Decimal(0.0002)

        cost = scatto_risposta + (self.billsec * sec_cost)
        return cost, currency

    def save(self,*args, **kwargs):
        if not self.callcost and self.dst.startswith("9"):
            callcost, currency = self.get_cost_eval()
            self.callcost= callcost
            self.callcost_currency = currency
        return super(Cdr, self).save(*args, **kwargs)




class Record (models.Model):
    record = models.AutoField(primary_key=True)
    ts = models.DateTimeField(
    )
    ts_date = models.DateField(
        blank = True, null = True,
        verbose_name = _("Record Date")
    )
    ts_time = models.TimeField(
        blank = True, null = True,
        verbose_name = _("Record Time")
    )
    caller = models.CharField(
        max_length = 80,
        verbose_name = _('Caller'),
        #help_text = 'Chiamante',
    )
    callee = models.CharField(
        max_length = 80,
        verbose_name = _('Caller Number'),
        #help_text = 'Caller*ID number',
    )
    recin = models.FileField(
        upload_to = RECORDS_PATH,
        blank = True, null = True,
        verbose_name = _('IN Record'),
        #help_text = 'Registrazione in Ingresso',
    )
    recout = models.FileField(
        upload_to = RECORDS_PATH,
        blank = True, null = True,
        verbose_name = _('OUT Record'),
        #help_text = 'Registrazione in Uscita',
    )
    recall = models.FileField(
        upload_to = RECORDS_PATH,
        verbose_name = _('Complete Record'),
        #help_text = 'Registrazione Completa',
    )
    country = models.CharField(
        max_length = 3,
        verbose_name = _("Country"),
        blank = True, null = True,
        help_text = _("Country of registration date")
    )


#    def get_local_ts(self):
#        tz = time.tzname[0]
#
#        # Se la telefonata e' stata inserita con l'ora italiana (country=ita)
#        # e la nazione su cui e' installata l'applicazione web e' inglese
#        # visualizzo la data con il relativo fuso. Stessa cosa per il contrario
#        if self.country == 'ita' and tz == 'GMT':
#            d = self.ts - timedelta(hours=1)
#            return formats.localize(d)
#
#        if self.country == 'uk' and tz == 'CET':
#            d = self.ts + timedelta(hours=1)
#            return formats.localize(d)
#
#        return formats.localize(self.ts)
#    get_local_ts.short_description = _("Record Datetime")


    class Meta:
        verbose_name = _("Record")
        verbose_name_plural = _("Records")
        ordering = ('-ts',)
        permissions = (
            ("view_record", "Can view record"),
        )


class AsteriskProfile(models.Model):
    user = models.ForeignKey (
        User,
        db_index = True,
        verbose_name = _("User"),
        blank=True, null=True
    )
    description = models.CharField(
        max_length=25,
        verbose_name = _('Description'),
        help_text = _('Phone Name that user see on interface'),
        blank=True, null=True
    )
    internal_number = models.CharField(
        max_length = 10,
        verbose_name = _('Internal Number')
    )

    def get_profile_name(self):
        # La descrizione vince su tutto. Se c'e, visualizzo quella
        if self.description:
            return self.description
        # Poi cerco l'utente, se c'e metto cognome nome puntato
        if self.user and self.user.last_name and self.user.first_name:
            return "%s %s." % (self.user.last_name, self.user.first_name[:1])

        if self.user and self.user.last_name:
            return "%s" % (self.user.last_name,)

        # se non trov niente visualizzo l'internal number
        return self.internal_number
    profile_name = property(get_profile_name)

    def __unicode__(self):
        if self.user:
            return u"%s (%s)" % (self.user.username, self.internal_number)
        if self.description:
            return u"%s (%s)" % (self.description, self.internal_number)
        return self.internal_number

    class Meta:
        verbose_name = _("Asterisk User Profile")
        verbose_name_plural = _("Asterisk User Profiles")
        permissions = (
            ("view_asteriskprofile", "Can view asterisk user profile"),
            ("view_status_phones", "Can view status phones"),
            ("make_calls", "Can Make Calls"),
        )

class AsteriskAddressBook(models.Model):
    description = models.CharField(
        max_length = 15,
        verbose_name = _('Description')
    )
    phone_number = models.CharField(
        max_length = 10,
        verbose_name = _('Phone Number')
    )
    user = models.ForeignKey (
        User,
        blank = True, null = True,
        verbose_name = _("User"),
    )

    is_internal = models.BooleanField()

    def __unicode__(self):
        if self.user:
            return u"%s -> %s (%s)" % (self.user.username, self.description, self.phone_number)
        else:
            return u"All Users -> %s (%s)" % (self.description, self.phone_number)

    class Meta:
        verbose_name = _("Asterisk address book")
        verbose_name_plural = _("Asterisk address book")
        permissions = (
            ("view_asteriskaddressbook", "Can view asterisk address book"),
        )

class Prefix(StatusModel, DateModel, UserModel):
    prefix = models.IntegerField()
    description = models.CharField(blank=True, null=True, max_length= 100	)

    def __unicode__(self):
        return u"%s" % str(self.prefix)

class Fare(StatusModel, DateModel, UserModel):
    prefix = models.ManyToManyField(Prefix, related_name='fares')
    name = models.CharField(max_length=100)
    connection_fee = models.DecimalField(max_digits=9, decimal_places=2, blank=True, null=True, help_text="'currency' al minuto")
    symbol_currency = models.CharField(max_length=5, default="€")

    def __unicode__(self):
        return u"%s" % self.name

class UnitCost(StatusModel, DateModel, UserModel):
    fare = models.ForeignKey(Fare, related_name='costs')
    name = models.CharField(max_length=100)
    cost = models.DecimalField(max_digits=9, decimal_places=2)
    hour_from = models.TimeField()
    hour_to = models.TimeField()

    def second_cost(self):
        a = self.cost / Decimal(60.0)


    def __unicode__(self):
        return u"%s - %s" % (self.fare, str(self.cost))


class Settings(JSettings):
    pass




#for i in Cdr.objects.all(): print i ; i.save()
