# -*- coding: utf-8 -*-
import time, datetime
import os, os.path
import telnetlib
import urllib2

from django.contrib.auth.decorators import login_required
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseNotFound, HttpResponseForbidden
from django.db import transaction
from django.views.generic.create_update import update_object, create_object
from django.utils.translation import ugettext_lazy as _
from django.core.urlresolvers import reverse
from django.utils.importlib import import_module
from django.db.models.aggregates import Sum, Count
from django.views.generic.list_detail import object_list

from jumbo.shortcuts import render_to_response, render_to_string
from jumbo.utils.functions import get_settings

from django.utils.datastructures import SortedDict

#from asterisk import settings
from asterisk.models import Cdr, Record, AsteriskProfile, UnitCost
from asterisk.models import AsteriskProfile, AsteriskAddressBook
from asterisk.utils import clean_number
from asterisk.forms import ReportForm


def get_formatted_time(s):
    ftime= "0s"
    m,s=divmod(s,60)
    h,m=divmod(m,60)
    d,h=divmod(h,24)
    if s > 0:
        ftime = "%ss" % s
    if m > 0:
        ftime = "%sm %s" % (m, ftime)
    if h > 0:
        ftime = "%sh %s" % (h, ftime)
    if d > 0:
        ftime = "%sg %s" % (m, ftime)

    return ftime

def get_nome_interno(id):
    nome = ""
    if id.find("<") > -1:
        nome = id.split("<")[0]
    return nome.strip()

def get_link_click_to_call(field):
    if field.isdigit():
        return "<a href='#' onclick='confirmCall(%s)' class='click_to_call'>%s</a>" % (field, field)
    return field

@login_required
def click_to_call(request, add_internal_prefix=True):
    HOST = get_settings("asterisk.settings", "ASTERISK_HOST")
    PORT = get_settings("asterisk.settings", "ASTERISK_PORT")
    user = get_settings("asterisk.settings", "ASTERISK_USER")
    password = get_settings("asterisk.settings", "ASTERISK_PWD")

    #------------------------------------------------------------------------------------------
    #edit the below variable values to reflect your system/information
    #------------------------------------------------------------------------------------------
    #specify the channel (extension) you want to receive the call requests with
    #e.g. SIP/XXX, IAX2/XXXX, ZAP/XXXX, etc
    try:
        u = AsteriskProfile.objects.get(user=request.user)
        if not u.internal_number:
            return HttpResponse("You don't have any internal number associated. Please contact the administrator.")
    except:
        return HttpResponse("You don't have permissions to make calls")

    if request.POST.has_key('numero1'):
        strChannel = "SIP/%s" % str(request.POST['numero1'])
    else:
        strChannel = "SIP/%s" % str(u.internal_number)

    #specify the context to make the outgoing call from.  By default, AAH uses from-internal
    #Using from-internal will make you outgoing dialing rules apply
    strContext = "from-internal"

    #specify the amount of time you want to try calling the specified channel before hangin up
    strWaitTime = "30"

    #specify the priority you wish to place on making this call
    strPriority = "1"

    #specify the maximum amount of retries
    strMaxRetry = "2"

    #--------------------------------------------------------------------------------------------
    #Shouldn't need to edit anything below this point to make this script work
    #--------------------------------------------------------------------------------------------
    #get the phone number from the posted form
    if request.POST.has_key('numero'):
        phoneNumber = clean_number(request.POST['numero'], add_internal_prefix=add_internal_prefix)
    elif request.GET.has_key('numero'):
        phoneNumber = clean_number(request.GET['numero'], add_internal_prefix=add_internal_prefix)
    else:
        phoneNumber = '0'

    #specify the caller id for the call
    strCallerId = "%s <%s>" % (str(u.internal_number), str(u.user.username))

    tn = telnetlib.Telnet(HOST, PORT)

    #tn.read_until("login: ")
    #tn.write(user + "\n")
    #if password:
    #    tn.read_until("Password: ")
    #    tn.write(password + "\n")

    tn.write("Action: login\r\n")
    tn.write("Events: off\r\n")
    tn.write("Username: %s\r\n" % user)
    tn.write("Secret: %s\r\n\r\n" % password)
    tn.write("Action: originate\r\n")
    tn.write("Channel: %s\r\n" % strChannel)
    tn.write("WaitTime: %s\r\n" % strWaitTime)
    tn.write("CallerId: %s\r\n" % strCallerId)
    tn.write("Exten: %s\r\n" % phoneNumber)
    tn.write("Context: %s\r\n" % strContext)
    tn.write("Priority: %s\r\n\r\n" % strPriority)
    tn.write("Action: Logoff\r\n\r\n")
    time.sleep(2)
    tn.close()
    return HttpResponse("OK")


@login_required
def download_registrazione(request, record_id):
    if request.user.is_staff:
        try:
            r = Record.objects.get(record=record_id)
            if not r.recall or not os.path.exists(r.recall):
                raise Exception
            a = open(r.recall)
            response = HttpResponse(a, mimetype='audio/x-wav')
            response['Content-Disposition'] = 'attachment; filename=%s' % r.recall.split('/')[-1]
            return response
        except:
            return HttpResponseNotFound('<h1>Registrazione inesistente</h1>')
    else:
        return HttpResponseForbidden("<h1>Accesso Negato</h1>")


@login_required
@transaction.commit_manually
def delete_registrazione(request, record_id):
    if request.user.is_staff:
        try:
            r = Record.objects.get(record=record_id)
            if not r.recall or not os.path.exists(r.recall):
                raise Exception

            f_record = r.recall

            # Cancellazione della registrazione dal database
            #
            r.delete()

            # Cancellazione della registrazione dal disco fisso
            #
            os.remove(f_record)
        except:
            transaction.rollback()
            raise Exception(e)
        else:
            transaction.commit()
            return HttpResponse("OK")
    else:
        return HttpResponseForbidden("<h1>Accesso Negato</h1>")


#@login_required
#def object_cdr_list(request):
    #if request.GET:
        #search_form = CdrSearchForm(request.GET)
        #data = request.GET.get('data', '')
    #else:
        #search_form = CdrSearchForm()

    #return render_to_response('asterisk/cdr_list.html', search_form=search_form)


@login_required
def make_call(request):
    return render_to_response('asterisk/make_call.html')

@login_required
def status_phones(request):
    if not request.user.has_perm("asterisk.view_status_phones"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    small_view = get_settings('asterisk.settings', 'ASTERISK_SMALL_VIEW')
    list_users = AsteriskProfile.objects.all().order_by('user__last_name', 'user__first_name', 'user__username')
    other_numbers = AsteriskAddressBook.objects.all().order_by('description')
    prefix_external = get_settings('asterisk.settings', 'ASTERISK_PREFIX_EXTERNAL')
    #current_user = AsteriskProfile.objects.filter(user__username=request.user.username)[0]
    return render_to_response('asterisk/status_phones.html', small_view=small_view,
                              list_users=list_users, other_numbers=other_numbers, prefix_external=prefix_external)

@login_required
def status_phones_popup(request):
    if not request.user.has_perm("asterisk.view_status_phones"):
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    small_view = get_settings('asterisk.settings', 'ASTERISK_SMALL_VIEW')
    list_users = AsteriskProfile.objects.all().order_by('user__last_name', 'user__first_name', 'user__username')
    other_numbers = AsteriskAddressBook.objects.all().order_by('description')
    prefix_external = get_settings('asterisk.settings', 'ASTERISK_PREFIX_EXTERNAL')
    #current_user = AsteriskProfile.objects.filter(user__username=request.user.username)[0]
    return render_to_response('asterisk/status_phones_popup.html', small_view=small_view,
                              list_users=list_users, other_numbers=other_numbers, prefix_external=prefix_external)

@login_required
def login(request):
    pwd = urllib2.quote(settings.ASTERISK_PWD, '^')
    url = 'http://%(host)s:%(port)s/asterisk/rawman/?action=login&username=%(username)s&secret=%(pwd)s' % {
                                                        'host': settings.ASTERISK_HOST,
                                                        'port': settings.ASTERISK_WEBSERVER_PORT,
                                                        'username': settings.ASTERISK_USER,
                                                        'pwd': pwd }
    r = urllib2.urlopen(url)
    html = r.read()
    response = HttpResponse(mimetype='text/plain')
    #response['Set-Cookie'] = r.headers['Set-Cookie']
    response.write(html)
    return response

@login_required
def status_line(request):
    return render_to_response('asterisk/status_line.html')

@login_required
def get_search_data(request, model):
    user = request.user
    get =  request.GET.copy()
    search_form = None

    if model == 'cdr':
        model_s = CdrSearch()
        search_form = CdrSearchForm(request.GET)
    elif model == 'record':
        model_s = RecordSearch()

    start = get.get('start', 0)
    limit = get.get('limit', 100)
    groupby = get.get('groupBy', '')
    orderby = get.get('sort', model_s.default_sort)
    direction = get.get('dir', 'DESC')

    queryset = model_s.get_query_set()

    # Gestione della ricerca
    #
    if search_form:
        search_form = CdrSearchForm(request.GET)
        queryset = queryset.filter(**search_form.filter())

    total = queryset.count()
    queryset = model_s.order_by(queryset, groupby, orderby, direction)
    queryset = queryset[int(start):int(start)+int(limit)]

    dati = model_s.get_dati(queryset)

    data = '{"success": true, "total": %s, "data": %s}' % (total, simplejson.dumps(dati, ensure_ascii=False))
    return HttpResponse(data, mimetype='text/javascript;')


class CdrSearch(object):
    default_sort = 'calldate'

    def get_query_set(self):
        queryset = Cdr.objects.all()
        return queryset

    def order_by(self, queryset, groupby, orderby, dir):
        if dir == 'DESC':
            orderby = "-%s" % orderby

        if groupby:
            return queryset.order_by(groupby, orderby)
        else:
            return queryset.order_by(orderby)

    def change_filter(self, filter_cnd):
        if filter_cnd.has_key('calldate__icontains'):
            d = filter_cnd['calldate__icontains']
            del filter_cnd['calldate__icontains']
            filter_cnd['calldate__year'] = d.year
            filter_cnd['calldate__month'] = d.month
            filter_cnd['calldate__day'] = d.day

        return filter_cnd

    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.id
            d['calldate'] = x.calldate.strftime("%m/%d/%Y %H:%M")
            d['calldate_for_group'] = x.calldate.strftime("%m/%d/%Y")
            nome = get_nome_interno(x.clid)
            src = get_link_click_to_call(x.src)
            if nome:
                d['src'] = "%s (%s)" % (src, nome)
            else:
                d['src'] = src
            d['dst'] = x.dst
            d['dcontext'] = x.get_dcontext_display()
            d['duration'] = get_formatted_time(x.duration)
            d['billsec'] = get_formatted_time(x.billsec)
            d['disposition'] = x.disposition
            dati.append(d)
        return dati

class RecordSearch(object):
    default_sort = 'ts'

    def get_query_set(self):
        queryset = Record.objects.all()
        return queryset

    def order_by(self, queryset, groupby, orderby, dir):
        if dir == 'DESC':
            orderby = "-%s" % orderby

        if groupby:
            return queryset.order_by(groupby, orderby)
        else:
            return queryset.order_by(orderby)

    def change_filter(self, filter_cnd):
        if filter_cnd.has_key('ts__icontains'):
            d = filter_cnd['ts__icontains']
            del filter_cnd['ts__icontains']
            filter_cnd['ts__year'] = d.year
            filter_cnd['ts__month'] = d.month
            filter_cnd['ts__day'] = d.day

        return filter_cnd

    def get_dati(self, queryset):
        dati = []
        for x in queryset:
            d = {}
            d['id'] = x.record
            d['ts'] = x.ts.strftime("%m/%d/%Y %H:%M")
            d['ts_for_group'] = x.ts.strftime("%m/%d/%Y")
            d['caller'] = x.caller
            d['callee'] = x.callee
            d['recin'] = x.recin
            d['recout'] = x.recout
            if x.recall:
                d['recall'] = x.recall.split('/')[-1]
            else:
                d['recall'] = ''
            dati.append(d)
        return dati

def unicode_or_none(data):
    if data == "": return None
    return data

#from django_tables2 import tables , utils

def report_centro_costo_orig(request):
    form = ReportForm()
    qs = Cdr.objects.filter(dst__startswith="9")
    qs = qs.filter(src__regex=r'^\s*[0-9]{4}$')


    if request.method == 'POST':
        form = ReportForm(request.POST)
        format = "%Y/%m/%d"
        if form.is_valid():
            if not form.data['interno'] == "":
                src = form.data['interno']
                qs = qs.filter(src= src)
            if form.data['data_da']:
                dateda = unicode_or_none( form.data['data_da'] )

            if form.data['data_a']:


                datea  = unicode_or_none( form.data['data_a'] )
                try:
                    da = datetime.datetime.strptime(dateda,format).date()
                except:
                    da = datetime.date.today()
                try:
                    a = datetime.datetime.strptime(datea,format).date()
                except:
                    a = datetime.date.today()

                qs = qs.filter( calldate_date__range=(da,a) )

    qs = qs.values('src').annotate(num_call= Count('src'),
                                   total_call_time = Sum('billsec'),
                                   total_cost = Sum('callcost'),
                                   clid = Count('clid'),
                                   )

    #table =  tables.Table(qs)
    #table._sequence= utils.Sequence(["src","clid"])
    #table.base_columns = {'src':1, 'clid':1}

    return render_to_response("admin/asterisk/fare/report.html", app_label="asterisk",
                              form=form, qs=qs)

def report_centro_costo(request):
    form = ReportForm()
    qs = Cdr.objects.filter(dst__startswith="9")

    if request.method == 'GET':
        form = ReportForm(request.GET)
        if form.is_valid():
            if form.cleaned_data.has_key('interno') and form.cleaned_data['interno']:
                qs = qs.filter(src=form.cleaned_data['interno'])

            if form.cleaned_data.has_key('data_da') and form.cleaned_data['data_da']:
                qs = qs.filter(calldate_date__gte=form.cleaned_data['data_da'])
            if form.cleaned_data.has_key('data_a') and form.cleaned_data['data_a']:
                qs = qs.filter(calldate_date__lte=form.cleaned_data['data_a'])

    qs = qs.values('src').annotate(num_call= Count('src'),
                           total_call_time = Sum('billsec'),
                           total_cost = Sum('callcost'),
                           ).order_by('src')

    return object_list(request, template_name = "admin/asterisk/fare/report.html",
         queryset = qs, paginate_by = 25, extra_context={'app_label':'asterisk', 'form':form})

#def callerid(request, phone):
#    callerid = import_module(settings.ASTERISK_CALLERID_MODULE)
#    try:
#        prefix = request.GET['prefix']
#    except:
#        prefix = ''
#    return HttpResponse(callerid.CallerID(phone, prefix))

#def proxy(request):
#    import urllib2
#    url = "%s?%s" % (request.GET['url'], request.GET['params'],)
#
#    f = urllib2.urlopen(url)
#    #data = "%s(%s)" % (callback, f.read())
#
#    return HttpResponse(f.read(), mimetype='text/javascript;')
